Package: ggplot2
Version: 3.4.0
Title: Create Elegant Data Visualisations Using the Grammar of Graphics
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = "aut",
           comment = c(ORCID = "0000-0003-4757-117X")),
    person("Winston", "Chang", role = "aut",
           comment = c(ORCID = "0000-0002-1576-2126")),
    person("Lionel", "Henry", role = "aut"),
    person("Thomas Lin", "Pedersen", , "thomas.pedersen@rstudio.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-5147-4711")),
    person("Kohske", "Takahashi", role = "aut"),
    person("Claus", "Wilke", role = "aut",
           comment = c(ORCID = "0000-0002-7470-9261")),
    person("Kara", "Woo", role = "aut",
           comment = c(ORCID = "0000-0002-5125-4188")),
    person("Hiroaki", "Yutani", role = "aut",
           comment = c(ORCID = "0000-0002-3385-7233")),
    person("Dewey", "Dunnington", role = "aut",
           comment = c(ORCID = "0000-0002-9415-4582")),
    person("RStudio", role = c("cph", "fnd"))
  )
Description: A system for 'declaratively' creating graphics, based on "The
    Grammar of Graphics". You provide the data, tell 'ggplot2' how to map
    variables to aesthetics, what graphical primitives to use, and it
    takes care of the details.
License: MIT + file LICENSE
URL: https://ggplot2.tidyverse.org,
        https://github.com/tidyverse/ggplot2
BugReports: https://github.com/tidyverse/ggplot2/issues
Depends: R (>= 3.3)
Imports: cli, glue, grDevices, grid, gtable (>= 0.1.1), isoband,
        lifecycle (> 1.0.1), MASS, mgcv, rlang (>= 1.0.0), scales (>=
        1.2.0), stats, tibble, vctrs (>= 0.5.0), withr (>= 2.5.0)
Suggests: covr, dplyr, ggplot2movies, hexbin, Hmisc, knitr, lattice,
        mapproj, maps, maptools, multcomp, munsell, nlme, profvis,
        quantreg, ragg, RColorBrewer, rgeos, rmarkdown, rpart, sf (>=
        0.7-3), svglite (>= 1.2.0.9001), testthat (>= 3.1.2), vdiffr
        (>= 1.0.0), xml2
Enhances: sp
VignetteBuilder: knitr
Config/Needs/website: ggtext, tidyr, forcats, tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.1
Collate: 'ggproto.r' 'ggplot-global.R' 'aaa-.r'
        'aes-colour-fill-alpha.r' 'aes-evaluation.r'
        'aes-group-order.r' 'aes-linetype-size-shape.r'
        'aes-position.r' 'compat-plyr.R' 'utilities.r' 'aes.r'
        'legend-draw.r' 'geom-.r' 'annotation-custom.r'
        'annotation-logticks.r' 'geom-polygon.r' 'geom-map.r'
        'annotation-map.r' 'geom-raster.r' 'annotation-raster.r'
        'annotation.r' 'autolayer.r' 'autoplot.r' 'axis-secondary.R'
        'backports.R' 'bench.r' 'bin.R' 'coord-.r' 'coord-cartesian-.r'
        'coord-fixed.r' 'coord-flip.r' 'coord-map.r' 'coord-munch.r'
        'coord-polar.r' 'coord-quickmap.R' 'coord-sf.R'
        'coord-transform.r' 'data.R' 'facet-.r' 'facet-grid-.r'
        'facet-null.r' 'facet-wrap.r' 'fortify-lm.r' 'fortify-map.r'
        'fortify-multcomp.r' 'fortify-spatial.r' 'fortify.r' 'stat-.r'
        'geom-abline.r' 'geom-rect.r' 'geom-bar.r' 'geom-bin2d.r'
        'geom-blank.r' 'geom-boxplot.r' 'geom-col.r' 'geom-path.r'
        'geom-contour.r' 'geom-count.r' 'geom-crossbar.r'
        'geom-segment.r' 'geom-curve.r' 'geom-defaults.r'
        'geom-ribbon.r' 'geom-density.r' 'geom-density2d.r'
        'geom-dotplot.r' 'geom-errorbar.r' 'geom-errorbarh.r'
        'geom-freqpoly.r' 'geom-function.R' 'geom-hex.r'
        'geom-histogram.r' 'geom-hline.r' 'geom-jitter.r'
        'geom-label.R' 'geom-linerange.r' 'geom-point.r'
        'geom-pointrange.r' 'geom-quantile.r' 'geom-rug.r' 'geom-sf.R'
        'geom-smooth.r' 'geom-spoke.r' 'geom-text.r' 'geom-tile.r'
        'geom-violin.r' 'geom-vline.r' 'ggplot2-package.R'
        'grob-absolute.r' 'grob-dotstack.r' 'grob-null.r' 'grouping.r'
        'guide-bins.R' 'guide-colorbar.r' 'guide-colorsteps.R'
        'guide-legend.r' 'guides-.r' 'guides-axis.r' 'guides-grid.r'
        'guides-none.r' 'hexbin.R' 'labeller.r' 'labels.r' 'layer.r'
        'layer-sf.R' 'layout.R' 'limits.r' 'margins.R' 'performance.R'
        'plot-build.r' 'plot-construction.r' 'plot-last.r' 'plot.r'
        'position-.r' 'position-collide.r' 'position-dodge.r'
        'position-dodge2.r' 'position-identity.r' 'position-jitter.r'
        'position-jitterdodge.R' 'position-nudge.R' 'position-stack.r'
        'quick-plot.r' 'range.r' 'reshape-add-margins.R' 'save.r'
        'scale-.r' 'scale-alpha.r' 'scale-binned.R' 'scale-brewer.r'
        'scale-colour.r' 'scale-continuous.r' 'scale-date.r'
        'scale-discrete-.r' 'scale-expansion.r' 'scale-gradient.r'
        'scale-grey.r' 'scale-hue.r' 'scale-identity.r'
        'scale-linetype.r' 'scale-linewidth.R' 'scale-manual.r'
        'scale-shape.r' 'scale-size.r' 'scale-steps.R' 'scale-type.R'
        'scale-view.r' 'scale-viridis.r' 'scales-.r' 'stat-align.R'
        'stat-bin.r' 'stat-bin2d.r' 'stat-bindot.r' 'stat-binhex.r'
        'stat-boxplot.r' 'stat-contour.r' 'stat-count.r'
        'stat-density-2d.r' 'stat-density.r' 'stat-ecdf.r'
        'stat-ellipse.R' 'stat-function.r' 'stat-identity.r'
        'stat-qq-line.R' 'stat-qq.r' 'stat-quantile.r'
        'stat-sf-coordinates.R' 'stat-sf.R' 'stat-smooth-methods.r'
        'stat-smooth.r' 'stat-sum.r' 'stat-summary-2d.r'
        'stat-summary-bin.R' 'stat-summary-hex.r' 'stat-summary.r'
        'stat-unique.r' 'stat-ydensity.r' 'summarise-plot.R'
        'summary.r' 'theme-elements.r' 'theme.r' 'theme-defaults.r'
        'theme-current.R' 'utilities-break.r' 'utilities-grid.r'
        'utilities-help.r' 'utilities-matrix.r'
        'utilities-resolution.r' 'utilities-table.r'
        'utilities-tidy-eval.R' 'zxx.r' 'zzz.r'
NeedsCompilation: no
Packaged: 2022-10-31 13:29:43 UTC; thomas
Author: Hadley Wickham [aut] (<https://orcid.org/0000-0003-4757-117X>),
  Winston Chang [aut] (<https://orcid.org/0000-0002-1576-2126>),
  Lionel Henry [aut],
  Thomas Lin Pedersen [aut, cre]
    (<https://orcid.org/0000-0002-5147-4711>),
  Kohske Takahashi [aut],
  Claus Wilke [aut] (<https://orcid.org/0000-0002-7470-9261>),
  Kara Woo [aut] (<https://orcid.org/0000-0002-5125-4188>),
  Hiroaki Yutani [aut] (<https://orcid.org/0000-0002-3385-7233>),
  Dewey Dunnington [aut] (<https://orcid.org/0000-0002-9415-4582>),
  RStudio [cph, fnd]
Maintainer: Thomas Lin Pedersen <thomas.pedersen@rstudio.com>
Repository: CRAN
Date/Publication: 2022-11-04 15:20:09 UTC
