Package: nstest
Type: Package
Title: Test ggplot2 NAMESPACE
Version: 0.1
Author: Hadley Wickham <h.wickham@gmail.com>
Maintainer: Hadley Wickham <h.wickham@gmail.com>
Description: Check to see if ggplot2 namespace set up correctly.
Imports:
    ggplot2
License: GPL-2
Collate:
    'my-plot.r'
