#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import scales grid gtable rlang vctrs
#' @importFrom glue glue glue_collapse
#' @importFrom lifecycle deprecated
#' @importFrom stats setNames
## usethis namespace: end
NULL
