# coord_polar throws informative warning about guides

    `coord_polar()` cannot render guide for the aesthetic: theta.

# coord_radial warns about axes

    `guide_axis()` cannot be used for theta.
    i Use one of x, y, or r instead.

---

    No appropriate placement found for outside r.axis.
    i Will use `r.axis.inside = TRUE` instead

# when both x and y are AsIs, they are not transformed

    `coord_radial()` cannot respect the <AsIs> class of `x` when `y` is not also <AsIs>.

