# type argument is checked for proper input

    The `type` argument must return a continuous scale for the colour aesthetic.
    x The provided object is not a scale function.

---

    The `type` argument must return a continuous scale for the fill aesthetic.
    x The provided object is not a scale function.

---

    The `type` argument must return a continuous scale for the colour aesthetic.
    x The provided scale works with the following aesthetics: fill and point_colour.

---

    The `type` argument must return a continuous scale for the fill aesthetic.
    x The provided scale is discrete.

---

    Unknown scale type: "abc"

---

    Unknown scale type: "abc"

