# coord_polar throws informative warning about guides

    `coord_polar()` cannot render guide for the aesthetic: theta.

# coord_radial warns about axes

    `guide_axis()` cannot be used for theta.
    i Use one of x, y, or r instead.

---

    No appropriate placement found for `r_axis_inside`.
    i Axis will be placed at panel edge.

