# check_required_aesthetics() errors on missing

    `test()` requires the following missing aesthetics: y.

---

    `test()` requires the following missing aesthetics: x and y.

---

    `test()` requires the following missing aesthetics: x or y.

---

    `test()` requires the following missing aesthetics: x and fill or y and fill.

# remove_missing checks input

    `na.rm` must be `TRUE` or `FALSE`, not an integer vector.

# tolower() and toupper() has been masked

    Please use `to_lower_ascii()`, which works fine in all locales.

---

    Please use `to_upper_ascii()`, which works fine in all locales.

# parse_safe() checks input

    `text` must be a character vector, not an integer vector.

# width_cm() and height_cm() checks input

    Don't know how to get width of <character> object

---

    Don't know how to get height of <character> object

# cut_*() checks its input and output

    Insufficient data values to produce 10 bins.

---

    Specify exactly one of `n` and `width`.

---

    Only one of `boundary` and `center` may be specified.

# interleave() checks the vector lengths

    Can't recycle `..1` (size 4) to match `..2` (size 0).

