# fortify.default proves a helpful error with class uneval

    `data` must be a <data.frame>, or an object coercible by `fortify()`, or a valid <data.frame>-like object coercible by `as.data.frame()`, not a <uneval> object.
    i Did you accidentally pass `aes()` to the `data` argument?

