# cartesian coords throws error when limits are badly specified

    `xlim` must be a vector of length 2, not a <ScaleContinuousPosition/ScaleContinuous/Scale/ggproto/gg> object.

---

    `ylim` must be a vector of length 2, not an integer vector of length 3.

