% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coord-.r
\name{is.Coord}
\alias{is.Coord}
\title{Is this object a coordinate system?}
\usage{
is.Coord(x)
}
\description{
Is this object a coordinate system?
}
\keyword{internal}
