\name{interleave}
\alias{interleave}
\title{Interleave (or zip) multiple vectors into a single vector.}
\usage{
  interleave(...)
}
\arguments{
  \item{...}{vectors to interleave}
}
\description{
  Interleave (or zip) multiple vectors into a single
  vector.
}
\keyword{internal}

