% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/facet-.r
\name{is.facet}
\alias{is.facet}
\title{Is this object a faceting specification?}
\usage{
is.facet(x)
}
\arguments{
\item{x}{object to test}
}
\description{
Is this object a faceting specification?
}
\keyword{internal}
