\name{absoluteGrob}
\alias{absoluteGrob}
\title{Absolute grob}
\usage{
  absoluteGrob(grob, width = NULL, height = NULL,
    xmin = NULL, ymin = NULL, vp = NULL)
}
\description{
  This grob has fixed dimesions and position.
}
\details{
  It's still experimental
}
\keyword{internal}

