% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-elements.r
\name{is.rel}
\alias{is.rel}
\title{Reports whether x is a rel object}
\usage{
is.rel(x)
}
\arguments{
\item{x}{An object to test}
}
\description{
Reports whether x is a rel object
}
\keyword{internal}
