% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggproto.r
\name{is.ggproto}
\alias{is.ggproto}
\title{Is an object a ggproto object?}
\usage{
is.ggproto(x)
}
\arguments{
\item{x}{An object to test.}
}
\description{
Is an object a ggproto object?
}

