\name{is.theme}
\alias{is.theme}
\title{Reports whether x is a theme object}
\usage{
  is.theme(x)
}
\arguments{
  \item{x}{An object to test}
}
\description{
  Reports whether x is a theme object
}

