% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.r
\name{is.theme}
\alias{is.theme}
\title{Reports whether x is a theme object}
\usage{
is.theme(x)
}
\arguments{
\item{x}{An object to test}
}
\description{
Reports whether x is a theme object
}
\keyword{internal}
