% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grob-null.r
\name{zeroGrob}
\alias{zeroGrob}
\title{The zero grob draws nothing and has zero size.}
\usage{
zeroGrob()
}
\description{
The zero grob draws nothing and has zero size.
}
\keyword{internal}
