\name{opts}
\alias{opts}
\title{Build a theme (or partial theme) from theme elements}
\usage{
  opts(...)
}
\arguments{
  \item{...}{Arguments to be passed on to the \code{theme}
  function.}
}
\description{
  \code{opts} is deprecated. See the \code{\link{theme}}
  function.
}

